@@ -4,9 +4,8 @@
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version. See http://www.gnu.org/copyleft/lgpl.html the full text of the
- * license.
+ * Software Foundation; either version 2 or version 3 of the License.
+ * See http://www.gnu.org/copyleft/lgpl.html the full text of the license.
  */
 
 #include <config.h>
@@ -46,16 +45,26 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 typedef struct
 {
-    gboolean connected;
+    /* Channel to write to daemon */
+    GIOChannel *to_server_channel;
 
-    GIOChannel *to_server_channel, *from_server_channel;
+    /* Channel to read from daemon */
+    GIOChannel *from_server_channel;
+
+    /* Data read from the daemon */
     guint8 *read_buffer;
     gsize n_read;
 
     gsize n_responses_waiting;
     GList *responses_received;
 
+    /* TRUE if have got a connect response */
+    gboolean connected;
+
+    /* Hints provided by the daemon */
     GHashTable *hints;
+
+    /* Timeout source to notify greeter to autologin */
     guint autologin_timeout;
 
     gchar *authentication_user;
@@ -82,7 +91,8 @@ typedef enum
     GREETER_MESSAGE_START_SESSION,
     GREETER_MESSAGE_CANCEL_AUTHENTICATION,
     GREETER_MESSAGE_SET_LANGUAGE,
-    GREETER_MESSAGE_AUTHENTICATE_REMOTE
+    GREETER_MESSAGE_AUTHENTICATE_REMOTE,
+    GREETER_MESSAGE_ENSURE_SHARED_DIR,
 } GreeterMessage;
 
 /* Messages from the server to the greeter */
@@ -91,7 +101,8 @@ typedef enum
     SERVER_MESSAGE_CONNECTED = 0,
     SERVER_MESSAGE_PROMPT_AUTHENTICATION,
     SERVER_MESSAGE_END_AUTHENTICATION,
-    SERVER_MESSAGE_SESSION_RESULT
+    SERVER_MESSAGE_SESSION_RESULT,
+    SERVER_MESSAGE_SHARED_DIR_RESULT,
 } ServerMessage;
 
 /**
@@ -102,7 +113,7 @@ typedef enum
  * Return value: the new #LightDMGreeter
  **/
 LightDMGreeter *
-lightdm_greeter_new ()
+lightdm_greeter_new (void)
 {
     return g_object_new (LIGHTDM_TYPE_GREETER, NULL);
 }
@@ -120,7 +131,7 @@ timed_login_cb (gpointer data)
 }
 
 static guint32
-int_length ()
+int_length (void)
 {
     return 4;
 }
@@ -218,7 +229,8 @@ static void
 write_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
 {
     LightDMGreeterPrivate *priv = GET_PRIVATE (greeter);
-    GIOStatus status;
+    gchar *data;
+    gsize data_length;
     GError *error = NULL;
     guint32 stated_length;
 
@@ -234,13 +246,29 @@ write_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
         return;
     }
 
-    status = g_io_channel_write_chars (priv->to_server_channel, (gchar *) message, message_length, NULL, &error);
+    data = (gchar *) message;
+    data_length = message_length;
+    while (data_length > 0)
+    {
+        GIOStatus status;
+        gsize n_written;
+
+        status = g_io_channel_write_chars (priv->to_server_channel, data, data_length, &n_written, &error);
+        if (error)
+            g_warning ("Error writing to daemon: %s", error->message);
+        g_clear_error (&error);
+        if (status != G_IO_STATUS_NORMAL)
+            return;
+
+        data_length -= n_written;
+        data += n_written;
+    }
+
+    g_debug ("Wrote %zi bytes to daemon", message_length);
+    g_io_channel_flush (priv->to_server_channel, &error);
     if (error)
-        g_warning ("Error writing to daemon: %s", error->message);
+        g_warning ("Failed to flush data to daemon: %s", error->message);
     g_clear_error (&error);
-    if (status == G_IO_STATUS_NORMAL)
-        g_debug ("Wrote %zi bytes to daemon", message_length);
-    g_io_channel_flush (priv->to_server_channel, NULL);
 }
 
 static void
@@ -414,7 +442,7 @@ read_message (LightDMGreeter *greeter, gsize *length, gboolean block)
 
     /* Stop if haven't got all the data we want */
     if (priv->n_read != n_to_read)
-        return FALSE;
+        return NULL;
 
     /* If have header, rerun for content */
     if (priv->n_read == HEADER_SIZE)
@@ -937,27 +965,23 @@ lightdm_greeter_respond (LightDMGreeter *greeter, const gchar *response)
 
     if (priv->n_responses_waiting == 0)
     {
-      guint32 msg_length;
-      GList *iter;
+        guint32 msg_length;
+        GList *iter;
 
-      g_debug ("Providing response to display manager");
+        g_debug ("Providing response to display manager");
 
-      msg_length = int_length ();
-      for (iter = priv->responses_received; iter; iter = iter->next)
-      {
-          msg_length += string_length ((gchar *)iter->data);
-      }
+        msg_length = int_length ();
+        for (iter = priv->responses_received; iter; iter = iter->next)
+            msg_length += string_length ((gchar *)iter->data);
 
-      write_header (message, MAX_MESSAGE_LENGTH, GREETER_MESSAGE_CONTINUE_AUTHENTICATION, msg_length, &offset);
-      write_int (message, MAX_MESSAGE_LENGTH, g_list_length (priv->responses_received), &offset);
-      for (iter = priv->responses_received; iter; iter = iter->next)
-      {
-          write_string (message, MAX_MESSAGE_LENGTH, (gchar *)iter->data, &offset);
-      }
-      write_message (greeter, message, offset);
+        write_header (message, MAX_MESSAGE_LENGTH, GREETER_MESSAGE_CONTINUE_AUTHENTICATION, msg_length, &offset);
+        write_int (message, MAX_MESSAGE_LENGTH, g_list_length (priv->responses_received), &offset);
+        for (iter = priv->responses_received; iter; iter = iter->next)
+            write_string (message, MAX_MESSAGE_LENGTH, (gchar *)iter->data, &offset);
+        write_message (greeter, message, offset);
 
-      g_list_free_full (priv->responses_received, g_free);
-      priv->responses_received = NULL;
+        g_list_free_full (priv->responses_received, g_free);
+        priv->responses_received = NULL;
     }
 }
 
@@ -1033,7 +1057,7 @@ lightdm_greeter_get_authentication_user (LightDMGreeter *greeter)
 /**
  * lightdm_greeter_set_language:
  * @greeter: A #LightDMGreeter
- * @language: The language to use for this user.
+ * @language: The language to use for this user in the form of a locale specification (e.g. "de_DE.UTF-8").
  *
  * Set the language for the currently authenticated user.
  **/
@@ -1107,6 +1131,70 @@ lightdm_greeter_start_session_sync (LightDMGreeter *greeter, const gchar *sessio
     return return_code == 0;
 }
 
+/**
+ * lightdm_greeter_ensure_shared_data_dir_sync:
+ * @greeter: A #LightDMGreeter
+ * @username: A username
+ *
+ * Ensure that a shared data dir for the given user is available.  Both the
+ * greeter user and @username will have write access to that folder.  The
+ * intention is that larger pieces of shared data would be stored there (files
+ * that the greeter creates but wants to give to a user -- like camera
+ * photos -- or files that the user creates but wants the greeter to
+ * see -- like contact avatars).
+ *
+ * LightDM will automatically create these if the user actually logs in, so
+ * greeters only need to call this method if they want to store something in
+ * the directory themselves.
+ *
+ * Return value: The path to the shared directory, free with g_free.
+ **/
+gchar *
+lightdm_greeter_ensure_shared_data_dir_sync (LightDMGreeter *greeter, const gchar *username)
+{
+    LightDMGreeterPrivate *priv;
+    guint8 message[MAX_MESSAGE_LENGTH];
+    guint8 *response;
+    gsize response_length, offset = 0;
+    guint32 id;
+    gchar *data_dir = NULL;
+
+    g_return_val_if_fail (LIGHTDM_IS_GREETER (greeter), NULL);
+
+    priv = GET_PRIVATE (greeter);
+
+    g_return_val_if_fail (priv->connected, NULL);
+
+    g_debug ("Ensuring data directory for user %s", username);
+
+    write_header (message, MAX_MESSAGE_LENGTH, GREETER_MESSAGE_ENSURE_SHARED_DIR, string_length (username), &offset);
+    write_string (message, MAX_MESSAGE_LENGTH, username, &offset);
+    write_message (greeter, message, offset);
+
+    response = read_message (greeter, &response_length, TRUE);
+    if (!response)
+        return NULL;
+
+    offset = 0;
+    id = read_int (response, response_length, &offset);
+    read_int (response, response_length, &offset);
+    if (id == SERVER_MESSAGE_SHARED_DIR_RESULT)
+        data_dir = read_string (response, response_length, &offset);
+    else
+        g_warning ("Expected SHARED_DIR_RESULT message, got %d", id);
+
+    /* Blank data dir means invalid user */
+    if (g_strcmp0 (data_dir, "") == 0)
+    {
+        g_free (data_dir);
+        data_dir = NULL;
+    }
+
+    g_free (response);
+
+    return data_dir;
+}
+
 static void
 lightdm_greeter_init (LightDMGreeter *greeter)
 {
@@ -1184,42 +1272,6 @@ lightdm_greeter_get_property (GObject    *object,
     }
 }
 
-static void
-marshal_VOID__STRING_INT (GClosure     *closure,
-                          GValue       *return_value G_GNUC_UNUSED,
-                          guint         n_param_values,
-                          const GValue *param_values,
-                          gpointer      invocation_hint G_GNUC_UNUSED,
-                          gpointer      marshal_data)
-{
-    typedef void (*GMarshalFunc_VOID__STRING_INT) (gpointer     data1,
-                                                   gpointer     arg_1,
-                                                   gint         arg_2,
-                                                   gpointer     data2);
-    register GMarshalFunc_VOID__STRING_INT callback;
-    register GCClosure *cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-
-    g_return_if_fail (n_param_values == 3);
-
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__STRING_INT) (marshal_data ? marshal_data : cc->callback);
-
-    callback (data1,
-              (param_values + 1)->data[0].v_pointer,
-              (param_values + 2)->data[0].v_int,
-              data2);
-}
-
 static void
 lightdm_greeter_finalize (GObject *object)
 {
@@ -1376,7 +1428,7 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (LightDMGreeterClass, show_prompt),
                       NULL, NULL,
-                      marshal_VOID__STRING_INT,
+                      NULL,
                       G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_INT);
 
     /**
@@ -1394,7 +1446,7 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (LightDMGreeterClass, show_message),
                       NULL, NULL,
-                      marshal_VOID__STRING_INT,
+                      NULL,
                       G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_INT);
 
     /**
@@ -1413,7 +1465,7 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (LightDMGreeterClass, authentication_complete),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
+                      NULL,
                       G_TYPE_NONE, 0);
 
     /**
@@ -1429,6 +1481,6 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (LightDMGreeterClass, autologin_timer_expired),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
+                      NULL,
                       G_TYPE_NONE, 0);
 }
