@@ -1,16 +1,15 @@
 /*
  * Copyright (C) 2010 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version. See http://www.gnu.org/copyleft/lgpl.html the full text of the
- * license.
+ * Software Foundation; either version 2 or version 3 of the License.
+ * See http://www.gnu.org/copyleft/lgpl.html the full text of the license.
  */
 
-#ifndef _LIGHTDM_LAYOUT_H_
-#define _LIGHTDM_LAYOUT_H_
+#ifndef LIGHTDM_LAYOUT_H_
+#define LIGHTDM_LAYOUT_H_
 
 #include <glib-object.h>
 
@@ -32,13 +31,13 @@ typedef struct
 {
     GObjectClass parent_class;
 
-	/* Reserved */
-	void (*reserved1) (void);
-	void (*reserved2) (void);
-	void (*reserved3) (void);
-	void (*reserved4) (void);
-	void (*reserved5) (void);
-	void (*reserved6) (void);
+    /* Reserved */
+    void (*reserved1) (void);
+    void (*reserved2) (void);
+    void (*reserved3) (void);
+    void (*reserved4) (void);
+    void (*reserved5) (void);
+    void (*reserved6) (void);
 } LightDMLayoutClass;
 
 GType lightdm_layout_get_type (void);
@@ -57,4 +56,4 @@ const gchar *lightdm_layout_get_description (LightDMLayout *layout);
 
 G_END_DECLS
 
-#endif /* _LIGHTDM_LAYOUT_H_ */
+#endif /* LIGHTDM_LAYOUT_H_ */
