@@ -5,9 +5,8 @@
  *
  * This library is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version. See http://www.gnu.org/copyleft/lgpl.html the full text of the
- * license.
+ * Software Foundation; either version 2 or version 3 of the License.
+ * See http://www.gnu.org/copyleft/lgpl.html the full text of the license.
  */
 
 #ifndef QLIGHTDM_POWER_H
@@ -35,16 +34,16 @@ namespace QLightDM
         bool canRestart();
 
     public Q_SLOTS:
-        void suspend();
-        void hibernate();
-        void shutdown();
-        void restart();
+        bool suspend();
+        bool hibernate();
+        bool shutdown();
+        bool restart();
 
     private:
         class PowerInterfacePrivate;
         PowerInterfacePrivate * const d;
 
     };
-};
+}
 
 #endif // QLIGHTDM_POWER_H
