@@ -15,14 +15,20 @@
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <gcrypt.h>
 
 #include "greeter.h"
-#include "ldm-marshal.h"
 #include "configuration.h"
+#include "shared-data-manager.h"
+
+enum {
+    PROP_0,
+    PROP_ACTIVE_USERNAME,
+};
 
 enum {
     CONNECTED,
-    START_AUTHENTICATION,
+    CREATE_SESSION,
     START_SESSION,
     LAST_SIGNAL
 };
@@ -30,9 +36,6 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 struct GreeterPrivate
 {
-    /* Session running on */
-    Session *session;
-
     /* PAM service to authenticate with */
     gchar *pam_service;
     gchar *autologin_pam_service;
@@ -40,6 +43,7 @@ struct GreeterPrivate
     /* Buffer for data read from greeter */
     guint8 *read_buffer;
     gsize n_read;
+    gboolean use_secure_memory;
   
     /* Hints for the greeter */
     GHashTable *hints;
@@ -53,6 +57,9 @@ struct GreeterPrivate
     /* Remote session name */
     gchar *remote_session;
 
+    /* Currently selected user */
+    gchar *active_username;
+
     /* PAM session being constructed by the greeter */
     Session *authentication_session;
 
@@ -66,11 +73,14 @@ struct GreeterPrivate
     gboolean guest_account_authenticated;
 
     /* Communication channels to communicate with */
+    int to_greeter_input;
+    int from_greeter_output;
     GIOChannel *to_greeter_channel;
     GIOChannel *from_greeter_channel;
+    guint from_greeter_watch;
 };
 
-G_DEFINE_TYPE (Greeter, greeter, G_TYPE_OBJECT);
+G_DEFINE_TYPE (Greeter, greeter, SESSION_TYPE);
 
 /* Messages from the greeter to the server */
 typedef enum
@@ -82,7 +92,8 @@ typedef enum
     GREETER_MESSAGE_START_SESSION,
     GREETER_MESSAGE_CANCEL_AUTHENTICATION,
     GREETER_MESSAGE_SET_LANGUAGE,
-    GREETER_MESSAGE_AUTHENTICATE_REMOTE
+    GREETER_MESSAGE_AUTHENTICATE_REMOTE,
+    GREETER_MESSAGE_ENSURE_SHARED_DIR,
 } GreeterMessage;
 
 /* Messages from the server to the greeter */
@@ -91,22 +102,25 @@ typedef enum
     SERVER_MESSAGE_CONNECTED = 0,
     SERVER_MESSAGE_PROMPT_AUTHENTICATION,
     SERVER_MESSAGE_END_AUTHENTICATION,
-    SERVER_MESSAGE_SESSION_RESULT
+    SERVER_MESSAGE_SESSION_RESULT,
+    SERVER_MESSAGE_SHARED_DIR_RESULT,
 } ServerMessage;
 
 static gboolean read_cb (GIOChannel *source, GIOCondition condition, gpointer data);
 
 Greeter *
-greeter_new (Session *session, const gchar *pam_service, const gchar *autologin_pam_service)
+greeter_new (void)
 {
-    Greeter *greeter;
+    return g_object_new (GREETER_TYPE, NULL);
+}
 
-    greeter = g_object_new (GREETER_TYPE, NULL);
-    greeter->priv->session = g_object_ref (session);
+void
+greeter_set_pam_services (Greeter *greeter, const gchar *pam_service, const gchar *autologin_pam_service)
+{
+    g_free (greeter->priv->pam_service);
     greeter->priv->pam_service = g_strdup (pam_service);
+    g_free (greeter->priv->autologin_pam_service);
     greeter->priv->autologin_pam_service = g_strdup (autologin_pam_service);
-
-    return greeter;
 }
 
 void
@@ -121,8 +135,35 @@ greeter_set_hint (Greeter *greeter, const gchar *name, const gchar *value)
     g_hash_table_insert (greeter->priv->hints, g_strdup (name), g_strdup (value));
 }
 
+static void *
+secure_malloc (Greeter *greeter, size_t n)
+{
+    if (greeter->priv->use_secure_memory)
+        return gcry_malloc_secure (n);
+    else
+        return g_malloc (n);
+}
+
+static void *
+secure_realloc (Greeter *greeter, void *ptr, size_t n)
+{
+    if (greeter->priv->use_secure_memory)
+        return gcry_realloc (ptr, n);
+    else
+        return g_realloc (ptr, n);
+}
+
+static void
+secure_free (Greeter *greeter, void *ptr)
+{
+    if (greeter->priv->use_secure_memory)
+        return gcry_free (ptr);
+    else
+        return g_free (ptr);
+}
+
 static guint32
-int_length ()
+int_length (void)
 {
     return 4;
 }
@@ -133,13 +174,31 @@ int_length ()
 static void
 write_message (Greeter *greeter, guint8 *message, gsize message_length)
 {
+    gchar *data;
+    gsize data_length;
     GError *error = NULL;
 
-    g_io_channel_write_chars (greeter->priv->to_greeter_channel, (gchar *) message, message_length, NULL, &error);
+    data = (gchar *) message;
+    data_length = message_length;
+    while (data_length > 0)
+    {
+        GIOStatus status;
+        gsize n_written;
+
+        status = g_io_channel_write_chars (greeter->priv->to_greeter_channel, data, data_length, &n_written, &error);
+        if (error)
+            l_warning (greeter, "Error writing to greeter: %s", error->message);
+        g_clear_error (&error);
+        if (status != G_IO_STATUS_NORMAL)
+            return;
+        data_length -= n_written;
+        data += n_written;
+    }
+
+    g_io_channel_flush (greeter->priv->to_greeter_channel, &error);
     if (error)
-        g_warning ("Error writing to greeter: %s", error->message);
+        l_warning (greeter, "Failed to flush data to greeter: %s", error->message);
     g_clear_error (&error);
-    g_io_channel_flush (greeter->priv->to_greeter_channel, NULL);
 }
 
 static void
@@ -198,7 +257,7 @@ handle_connect (Greeter *greeter, const gchar *version)
     GHashTableIter iter;
     gpointer key, value;
 
-    g_debug ("Greeter connected version=%s", version);
+    l_debug (greeter, "Greeter connected version=%s", version);
 
     length = string_length (VERSION);
     g_hash_table_iter_init (&iter, greeter->priv->hints);
@@ -233,7 +292,7 @@ pam_messages_cb (Session *session, Greeter *greeter)
     messages_length = session_get_messages_length (session);
 
     /* Respond to d-bus query with messages */
-    g_debug ("Prompt greeter with %d message(s)", messages_length);
+    l_debug (greeter, "Prompt greeter with %d message(s)", messages_length);
     size = int_length () + string_length (session_get_username (session)) + int_length ();
     for (i = 0; i < messages_length; i++)
         size += int_length () + string_length (messages[i].msg);
@@ -259,6 +318,7 @@ pam_messages_cb (Session *session, Greeter *greeter)
         struct pam_response *response;
         response = calloc (messages_length, sizeof (struct pam_response));
         session_respond (greeter->priv->authentication_session, response);
+        free (response);
     }
 }
 
@@ -280,16 +340,16 @@ authentication_complete_cb (Session *session, Greeter *greeter)
 {
     int result;
 
-    g_debug ("Authenticate result for user %s: %s", session_get_username (session), session_get_authentication_result_string (session));
+    l_debug (greeter, "Authenticate result for user %s: %s", session_get_username (session), session_get_authentication_result_string (session));
 
     result = session_get_authentication_result (session);
     if (session_get_is_authenticated (session))
     {
         if (session_get_user (session))
-            g_debug ("User %s authorized", session_get_username (session));
+            l_debug (greeter, "User %s authorized", session_get_username (session));
         else
         {
-            g_debug ("User %s authorized, but no account of that name exists", session_get_username (session));          
+            l_debug (greeter, "User %s authorized, but no account of that name exists", session_get_username (session));
             result = PAM_USER_UNKNOWN;
         }
     }
@@ -321,16 +381,21 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
 
     if (username[0] == '\0')
     {
-        g_debug ("Greeter start authentication");
+        l_debug (greeter, "Greeter start authentication");
         username = NULL;
     }
     else
-        g_debug ("Greeter start authentication for %s", username);
+        l_debug (greeter, "Greeter start authentication for %s", username);
 
     reset_session (greeter);
 
+    if (greeter->priv->active_username)
+        g_free (greeter->priv->active_username);
+    greeter->priv->active_username = g_strdup (username);
+    g_object_notify (G_OBJECT (greeter), "active-username");
+
     greeter->priv->authentication_sequence_number = sequence_number;
-    g_signal_emit (greeter, signals[START_AUTHENTICATION], 0, username, &greeter->priv->authentication_session);
+    g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
     if (!greeter->priv->authentication_session)
     {
         send_end_authentication (greeter, sequence_number, "", PAM_USER_UNKNOWN);
@@ -354,19 +419,23 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
     }
 
     /* Run the session process */
-    session_start (greeter->priv->authentication_session, service, username, TRUE, is_interactive, FALSE);
+    session_set_pam_service (greeter->priv->authentication_session, service);
+    session_set_username (greeter->priv->authentication_session, username);
+    session_set_do_authenticate (greeter->priv->authentication_session, TRUE);
+    session_set_is_interactive (greeter->priv->authentication_session, is_interactive);
+    session_start (greeter->priv->authentication_session);
 }
 
 static void
 handle_login_as_guest (Greeter *greeter, guint32 sequence_number)
 {
-    g_debug ("Greeter start authentication for guest account");
+    l_debug (greeter, "Greeter start authentication for guest account");
 
     reset_session (greeter);
 
     if (!greeter->priv->allow_guest)
     {
-        g_debug ("Guest account is disabled");
+        l_debug (greeter, "Guest account is disabled");
         send_end_authentication (greeter, sequence_number, "", PAM_USER_UNKNOWN);
         return;
     }
@@ -417,11 +486,11 @@ handle_login_remote (Greeter *greeter, const gchar *session_name, const gchar *u
 
     if (username[0] == '\0')
     {
-        g_debug ("Greeter start authentication for remote session %s", session_name);
+        l_debug (greeter, "Greeter start authentication for remote session %s", session_name);
         username = NULL;
     }
     else
-        g_debug ("Greeter start authentication for remote session %s as user %s", session_name, username);
+        l_debug (greeter, "Greeter start authentication for remote session %s as user %s", session_name, username);
 
     reset_session (greeter);
 
@@ -434,14 +503,19 @@ handle_login_remote (Greeter *greeter, const gchar *session_name, const gchar *u
 
     greeter->priv->authentication_sequence_number = sequence_number;
     greeter->priv->remote_session = g_strdup (session_name);
-    g_signal_emit (greeter, signals[START_AUTHENTICATION], 0, username, &greeter->priv->authentication_session);
+    g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
     if (greeter->priv->authentication_session)
     {
         g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "got-messages", G_CALLBACK (pam_messages_cb), greeter);
         g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "authentication-complete", G_CALLBACK (authentication_complete_cb), greeter);
 
         /* Run the session process */
-        session_start (greeter->priv->authentication_session, service, username, TRUE, TRUE, TRUE);
+        session_set_pam_service (greeter->priv->authentication_session, service);
+        session_set_username (greeter->priv->authentication_session, username);
+        session_set_do_authenticate (greeter->priv->authentication_session, TRUE);
+        session_set_is_interactive (greeter->priv->authentication_session, TRUE);
+        session_set_is_guest (greeter->priv->authentication_session, TRUE);
+        session_start (greeter->priv->authentication_session);
     }
 
     g_free (service);
@@ -481,7 +555,7 @@ handle_continue_authentication (Greeter *greeter, gchar **secrets)
         return;
     }
 
-    g_debug ("Continue authentication");
+    l_debug (greeter, "Continue authentication");
 
     /* Build response */
     response = calloc (messages_length, sizeof (struct pam_response));
@@ -490,12 +564,18 @@ handle_continue_authentication (Greeter *greeter, gchar **secrets)
         int msg_style = messages[i].msg_style;
         if (msg_style == PAM_PROMPT_ECHO_OFF || msg_style == PAM_PROMPT_ECHO_ON)
         {
-            response[i].resp = strdup (secrets[j]); // FIXME: Need to convert from UTF-8
+            size_t secret_length = strlen (secrets[j]) + 1;
+            response[i].resp = secure_malloc (greeter, secret_length);
+            memcpy (response[i].resp, secrets[j], secret_length); // FIXME: Need to convert from UTF-8
             j++;
         }
     }
 
     session_respond (greeter->priv->authentication_session, response);
+
+    for (i = 0; i < messages_length; i++)
+        secure_free (greeter, response[i].resp);
+    free (response);
 }
 
 static void
@@ -505,7 +585,7 @@ handle_cancel_authentication (Greeter *greeter)
     if (greeter->priv->authentication_session == NULL)
         return;
 
-    g_debug ("Cancel authentication");
+    l_debug (greeter, "Cancel authentication");
     reset_session (greeter);
 }
 
@@ -530,15 +610,15 @@ handle_start_session (Greeter *greeter, const gchar *session)
     if (greeter->priv->guest_account_authenticated || session_get_is_authenticated (greeter->priv->authentication_session))
     {
         if (session)
-            g_debug ("Greeter requests session %s", session);
+            l_debug (greeter, "Greeter requests session %s", session);
         else
-            g_debug ("Greeter requests default session");
+            l_debug (greeter, "Greeter requests default session");
         greeter->priv->start_session = TRUE;
         g_signal_emit (greeter, signals[START_SESSION], 0, session_type, session, &result);
     }
     else
     {
-        g_debug ("Ignoring start session request, user is not authorized");
+        l_debug (greeter, "Ignoring start session request, user is not authorized");
         result = FALSE;
     }
 
@@ -554,22 +634,40 @@ handle_set_language (Greeter *greeter, const gchar *language)
 
     if (!greeter->priv->guest_account_authenticated && !session_get_is_authenticated (greeter->priv->authentication_session))
     {
-        g_debug ("Ignoring set language request, user is not authorized");
+        l_debug (greeter, "Ignoring set language request, user is not authorized");
         return;
     }
 
     // FIXME: Could use this
     if (greeter->priv->guest_account_authenticated)
     {
-        g_debug ("Ignoring set language request for guest user");
+        l_debug (greeter, "Ignoring set language request for guest user");
         return;
     }
 
-    g_debug ("Greeter sets language %s", language);
+    l_debug (greeter, "Greeter sets language %s", language);
     user = session_get_user (greeter->priv->authentication_session);
     user_set_language (user, language);
 }
 
+static void
+handle_ensure_shared_dir (Greeter *greeter, const gchar *username)
+{
+    gchar *dir;
+    guint8 message[MAX_MESSAGE_LENGTH];
+    gsize offset = 0;
+
+    l_debug (greeter, "Greeter requests data directory for user %s", username);
+
+    dir = shared_data_manager_ensure_user_dir (shared_data_manager_get_instance (), username);
+
+    write_header (message, MAX_MESSAGE_LENGTH, SERVER_MESSAGE_SHARED_DIR_RESULT, string_length (dir), &offset);
+    write_string (message, MAX_MESSAGE_LENGTH, dir, &offset);
+    write_message (greeter, message, offset);
+
+    g_free (dir);
+}
+
 static guint32
 read_int (Greeter *greeter, gsize *offset)
 {
@@ -577,7 +675,7 @@ read_int (Greeter *greeter, gsize *offset)
     guint8 *buffer;
     if (greeter->priv->n_read - *offset < sizeof (guint32))
     {
-        g_warning ("Not enough space for int, need %zu, got %zu", sizeof (guint32), greeter->priv->n_read - *offset);
+        l_warning (greeter, "Not enough space for int, need %zu, got %zu", sizeof (guint32), greeter->priv->n_read - *offset);
         return 0;
     }
     buffer = greeter->priv->read_buffer + *offset;
@@ -586,8 +684,26 @@ read_int (Greeter *greeter, gsize *offset)
     return value;
 }
 
+static int
+get_message_length (Greeter *greeter)
+{
+    gsize offset;
+    int payload_length;
+
+    offset = int_length ();
+    payload_length = read_int (greeter, &offset);
+
+    if (HEADER_SIZE + payload_length < HEADER_SIZE)
+    {
+        l_warning (greeter, "Payload length of %u octets too long", payload_length);
+        return HEADER_SIZE;
+    }
+
+    return HEADER_SIZE + payload_length;
+}
+
 static gchar *
-read_string (Greeter *greeter, gsize *offset)
+read_string_full (Greeter *greeter, gsize *offset, void* (*alloc_fn)(size_t n))
 {
     guint32 length;
     gchar *value;
@@ -595,11 +711,11 @@ read_string (Greeter *greeter, gsize *offset)
     length = read_int (greeter, offset);
     if (greeter->priv->n_read - *offset < length)
     {
-        g_warning ("Not enough space for string, need %u, got %zu", length, greeter->priv->n_read - *offset);
+        l_warning (greeter, "Not enough space for string, need %u, got %zu", length, greeter->priv->n_read - *offset);
         return g_strdup ("");
     }
 
-    value = g_malloc (sizeof (gchar *) * (length + 1));
+    value = (*alloc_fn) (sizeof (gchar) * (length + 1));
     memcpy (value, greeter->priv->read_buffer + *offset, length);
     value[length] = '\0';
     *offset += length;
@@ -607,29 +723,49 @@ read_string (Greeter *greeter, gsize *offset)
     return value;
 }
 
+static gchar *
+read_string (Greeter *greeter, gsize *offset)
+{
+    return read_string_full (greeter, offset, g_malloc);
+}
+
+static gchar *
+read_secret (Greeter *greeter, gsize *offset)
+{
+    if (greeter->priv->use_secure_memory)
+        return read_string_full (greeter, offset, gcry_malloc_secure);
+    else
+        return read_string_full (greeter, offset, g_malloc);
+}
+
 static gboolean
 read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
 {
     Greeter *greeter = data;
     gsize n_to_read, n_read, offset;
     GIOStatus status;
-    int id, n_secrets, i;
-    guint32 sequence_number;
+    int id, i;
+    guint32 sequence_number, n_secrets, max_secrets;
     gchar *version, *username, *session_name, *language;
     gchar **secrets;
     GError *error = NULL;
 
     if (condition == G_IO_HUP)
     {
-        g_debug ("Greeter closed communication channel");
+        l_debug (greeter, "Greeter closed communication channel");
+        greeter->priv->from_greeter_watch = 0;
         return FALSE;
     }
   
     n_to_read = HEADER_SIZE;
     if (greeter->priv->n_read >= HEADER_SIZE)
     {
-        offset = int_length ();
-        n_to_read += read_int (greeter, &offset);
+        n_to_read = get_message_length (greeter);
+        if (n_to_read <= HEADER_SIZE)
+        {
+            greeter->priv->from_greeter_watch = 0;
+            return FALSE;
+        }
     }
 
     status = g_io_channel_read_chars (greeter->priv->from_greeter_channel,
@@ -638,7 +774,7 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
                                       &n_read,
                                       &error);
     if (error)
-        g_warning ("Error reading from greeter: %s", error->message);
+        l_warning (greeter, "Error reading from greeter: %s", error->message);
     g_clear_error (&error);
     if (status != G_IO_STATUS_NORMAL)
         return TRUE;
@@ -650,14 +786,13 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
     /* If have header, rerun for content */
     if (greeter->priv->n_read == HEADER_SIZE)
     {
-        gsize offset = int_length ();
-        n_to_read = read_int (greeter, &offset);
-        if (n_to_read > 0)
+        n_to_read = get_message_length (greeter);
+        if (n_to_read > HEADER_SIZE)
         {
-            greeter->priv->read_buffer = g_realloc (greeter->priv->read_buffer, HEADER_SIZE + n_to_read);
+            greeter->priv->read_buffer = secure_realloc (greeter, greeter->priv->read_buffer, n_to_read);
             read_cb (source, condition, greeter);
             return TRUE;
-        }
+        }      
     }
   
     offset = 0;
@@ -688,12 +823,21 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
         break;
     case GREETER_MESSAGE_CONTINUE_AUTHENTICATION:
         n_secrets = read_int (greeter, &offset);
+        max_secrets = (G_MAXUINT32 - 1) / sizeof (gchar *);
+        if (n_secrets > max_secrets)
+        {
+            l_warning (greeter, "Array length of %u elements too long", n_secrets);
+            greeter->priv->from_greeter_watch = 0;
+            return FALSE;
+        }
         secrets = g_malloc (sizeof (gchar *) * (n_secrets + 1));
         for (i = 0; i < n_secrets; i++)
-            secrets[i] = read_string (greeter, &offset);
+            secrets[i] = read_secret (greeter, &offset);
         secrets[i] = NULL;
         handle_continue_authentication (greeter, secrets);
-        g_strfreev (secrets);
+        for (i = 0; i < n_secrets; i++)
+            secure_free (greeter, secrets[i]);
+        g_free (secrets);
         break;
     case GREETER_MESSAGE_CANCEL_AUTHENTICATION:
         handle_cancel_authentication (greeter);
@@ -708,8 +852,13 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
         handle_set_language (greeter, language);
         g_free (language);
         break;
+    case GREETER_MESSAGE_ENSURE_SHARED_DIR:
+        username = read_string (greeter, &offset);
+        handle_ensure_shared_dir (greeter, username);
+        g_free (username);
+        break;
     default:
-        g_warning ("Unknown message from greeter: %d", id);
+        l_warning (greeter, "Unknown message from greeter: %d", id);
         break;
     }
 
@@ -739,10 +888,19 @@ greeter_get_start_session (Greeter *greeter)
     return greeter->priv->start_session;
 }
 
-gboolean
-greeter_start (Greeter *greeter, const gchar *service, const gchar *username)
+const gchar *
+greeter_get_active_username (Greeter *greeter)
 {
+    g_return_val_if_fail (greeter != NULL, NULL);
+    return greeter->priv->active_username;
+}
+
+static gboolean
+greeter_start (Session *session)
+{
+    Greeter *greeter = GREETER (session);
     int to_greeter_pipe[2], from_greeter_pipe[2];
+    int to_greeter_output, from_greeter_input;
     gboolean result = FALSE;
     gchar *value;
 
@@ -752,36 +910,40 @@ greeter_start (Greeter *greeter, const gchar *service, const gchar *username)
         g_warning ("Failed to create pipes: %s", strerror (errno));
         return FALSE;
     }
-    greeter->priv->to_greeter_channel = g_io_channel_unix_new (to_greeter_pipe[1]);
+    to_greeter_output = to_greeter_pipe[0];
+    greeter->priv->to_greeter_input = to_greeter_pipe[1];
+    greeter->priv->to_greeter_channel = g_io_channel_unix_new (greeter->priv->to_greeter_input);
     g_io_channel_set_encoding (greeter->priv->to_greeter_channel, NULL, NULL);
-    greeter->priv->from_greeter_channel = g_io_channel_unix_new (from_greeter_pipe[0]);
+    greeter->priv->from_greeter_output = from_greeter_pipe[0];
+    from_greeter_input = from_greeter_pipe[1];
+    greeter->priv->from_greeter_channel = g_io_channel_unix_new (greeter->priv->from_greeter_output);
     g_io_channel_set_encoding (greeter->priv->from_greeter_channel, NULL, NULL);
     g_io_channel_set_buffered (greeter->priv->from_greeter_channel, FALSE);
-    g_io_add_watch (greeter->priv->from_greeter_channel, G_IO_IN | G_IO_HUP, read_cb, greeter);
+    greeter->priv->from_greeter_watch = g_io_add_watch (greeter->priv->from_greeter_channel, G_IO_IN | G_IO_HUP, read_cb, greeter);
 
     /* Let the greeter session know how to communicate with the daemon */
-    value = g_strdup_printf ("%d", from_greeter_pipe[1]);
-    session_set_env (greeter->priv->session, "LIGHTDM_TO_SERVER_FD", value);
+    value = g_strdup_printf ("%d", from_greeter_input);
+    session_set_env (SESSION (greeter), "LIGHTDM_TO_SERVER_FD", value);
     g_free (value);
-    value = g_strdup_printf ("%d", to_greeter_pipe[0]);
-    session_set_env (greeter->priv->session, "LIGHTDM_FROM_SERVER_FD", value);
+    value = g_strdup_printf ("%d", to_greeter_output);
+    session_set_env (SESSION (greeter), "LIGHTDM_FROM_SERVER_FD", value);
     g_free (value);
 
     /* Don't allow the daemon end of the pipes to be accessed in child processes */
-    fcntl (to_greeter_pipe[1], F_SETFD, FD_CLOEXEC);
-    fcntl (from_greeter_pipe[0], F_SETFD, FD_CLOEXEC);
+    fcntl (greeter->priv->to_greeter_input, F_SETFD, FD_CLOEXEC);
+    fcntl (greeter->priv->from_greeter_output, F_SETFD, FD_CLOEXEC);
 
-    result = session_start (greeter->priv->session, service, username, FALSE, FALSE, FALSE);
+    result = SESSION_CLASS (greeter_parent_class)->start (session);
 
     /* Close the session ends of the pipe */
-    close (to_greeter_pipe[0]);
-    close (from_greeter_pipe[1]);
+    close (to_greeter_output);
+    close (from_greeter_input);
 
     return result;
 }
 
 static Session *
-greeter_real_start_authentication (Greeter *greeter, const gchar *username)
+greeter_real_create_session (Greeter *greeter)
 {
     return NULL;
 }
@@ -792,12 +954,27 @@ greeter_real_start_session (Greeter *greeter, SessionType type, const gchar *ses
     return FALSE;
 }
 
+static void
+greeter_stop (Session *session)
+{
+    Greeter *greeter = GREETER (session);
+
+    /* Stop any events occurring after we've stopped */
+    if (greeter->priv->authentication_session)
+        g_signal_handlers_disconnect_matched (greeter->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, greeter);
+
+    SESSION_CLASS (greeter_parent_class)->stop (session);
+}
+
 static void
 greeter_init (Greeter *greeter)
 {
     greeter->priv = G_TYPE_INSTANCE_GET_PRIVATE (greeter, GREETER_TYPE, GreeterPrivate);
-    greeter->priv->read_buffer = g_malloc (HEADER_SIZE);
+    greeter->priv->read_buffer = secure_malloc (greeter, HEADER_SIZE);
     greeter->priv->hints = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+    greeter->priv->use_secure_memory = config_get_boolean (config_get_instance (), "LightDM", "lock-memory");
+    greeter->priv->to_greeter_input = -1;
+    greeter->priv->from_greeter_output = -1;
 }
 
 static void
@@ -807,34 +984,69 @@ greeter_finalize (GObject *object)
 
     self = GREETER (object);
 
-    g_signal_handlers_disconnect_matched (self->priv->session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);    
-    g_object_unref (self->priv->session);
     g_free (self->priv->pam_service);
     g_free (self->priv->autologin_pam_service);
-    g_free (self->priv->read_buffer);
+    secure_free (self, self->priv->read_buffer);
     g_hash_table_unref (self->priv->hints);
     g_free (self->priv->remote_session);
+    g_free (self->priv->active_username);
     if (self->priv->authentication_session)
     {
         g_signal_handlers_disconnect_matched (self->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
         g_object_unref (self->priv->authentication_session);
     }
+    close (self->priv->to_greeter_input);
+    close (self->priv->from_greeter_output);
     if (self->priv->to_greeter_channel)
         g_io_channel_unref (self->priv->to_greeter_channel);
     if (self->priv->from_greeter_channel)
         g_io_channel_unref (self->priv->from_greeter_channel);
+    if (self->priv->from_greeter_watch)
+        g_source_remove (self->priv->from_greeter_watch);
 
     G_OBJECT_CLASS (greeter_parent_class)->finalize (object);
 }
 
+static void
+greeter_set_property (GObject      *object,
+                      guint         prop_id,
+                      const GValue *value,
+                      GParamSpec   *pspec)
+{
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+}
+
+static void
+greeter_get_property (GObject    *object,
+                      guint       prop_id,
+                      GValue     *value,
+                      GParamSpec *pspec)
+{
+    Greeter *greeter = GREETER (object);
+
+    switch (prop_id) {
+    case PROP_ACTIVE_USERNAME:
+        g_value_set_string (value, greeter_get_active_username (greeter));
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
 static void
 greeter_class_init (GreeterClass *klass)
 {
+    SessionClass *session_class = SESSION_CLASS (klass);
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-    klass->start_authentication = greeter_real_start_authentication;
+    klass->create_session = greeter_real_create_session;
     klass->start_session = greeter_real_start_session;
+    session_class->start = greeter_start;
+    session_class->stop = greeter_stop;
     object_class->finalize = greeter_finalize;
+    object_class->get_property = greeter_get_property;
+    object_class->set_property = greeter_set_property;
 
     signals[CONNECTED] =
         g_signal_new ("connected",
@@ -842,18 +1054,18 @@ greeter_class_init (GreeterClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, connected),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
+                      NULL,
                       G_TYPE_NONE, 0);
 
-    signals[START_AUTHENTICATION] =
-        g_signal_new ("start-authentication",
+    signals[CREATE_SESSION] =
+        g_signal_new ("create-session",
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (GreeterClass, start_authentication),
+                      G_STRUCT_OFFSET (GreeterClass, create_session),
                       g_signal_accumulator_first_wins,
                       NULL,
-                      ldm_marshal_OBJECT__STRING,
-                      SESSION_TYPE, 1, G_TYPE_STRING);
+                      NULL,
+                      SESSION_TYPE, 0);
 
     signals[START_SESSION] =
         g_signal_new ("start-session",
@@ -862,8 +1074,16 @@ greeter_class_init (GreeterClass *klass)
                       G_STRUCT_OFFSET (GreeterClass, start_session),
                       g_signal_accumulator_true_handled,
                       NULL,
-                      ldm_marshal_BOOLEAN__INT_STRING,
+                      NULL,
                       G_TYPE_BOOLEAN, 2, G_TYPE_INT, G_TYPE_STRING);
 
+    g_object_class_install_property (object_class,
+                                     PROP_ACTIVE_USERNAME,
+                                     g_param_spec_string ("active-username",
+                                                          "active-username",
+                                                          "Active username",
+                                                          NULL,
+                                                          G_PARAM_READABLE));
+
     g_type_class_add_private (klass, sizeof (GreeterPrivate));
 }
