@@ -9,8 +9,8 @@
  * license.
  */
 
-#ifndef _VNC_SERVER_H_
-#define _VNC_SERVER_H_
+#ifndef VNC_SERVER_H_
+#define VNC_SERVER_H_
 
 #include <glib-object.h>
 
@@ -42,8 +42,12 @@ void vnc_server_set_port (VNCServer *server, guint port);
 
 guint vnc_server_get_port (VNCServer *server);
 
+void vnc_server_set_listen_address (VNCServer *server, const gchar *listen_address);
+
+const gchar *vnc_server_get_listen_address (VNCServer *server);
+
 gboolean vnc_server_start (VNCServer *server);
 
 G_END_DECLS
 
-#endif /* _VNC_SERVER_H_ */
+#endif /* VNC_SERVER_H_ */
