@@ -9,11 +9,13 @@
  * license.
  */
 
-#ifndef _VT_H_
-#define _VT_H_
+#ifndef VT_H_
+#define VT_H_
 
 #include <glib-object.h>
 
+gboolean vt_can_multi_seat (void);
+
 gint vt_get_active (void);
 
 gint vt_get_unused (void);
@@ -26,4 +28,4 @@ void vt_unref (gint number);
 
 void vt_set_active (gint number);
 
-#endif /* _VT_H_ */
+#endif /* VT_H_ */
