@@ -27,9 +27,6 @@ struct XServerLocalPrivate
 {
     /* X server process */
     Process *x_server_process;
-  
-    /* File to log to */
-    gchar *log_file;    
 
     /* Command to run the X server */
     gchar *command;
@@ -43,6 +40,9 @@ struct XServerLocalPrivate
     /* Value for -seat argument */
     gchar *xdg_seat;
 
+    /* TRUE if should share VT with other X server */
+    gboolean sharevts;
+
     /* TRUE if TCP/IP connections are allowed */
     gboolean allow_tcp;
 
@@ -77,8 +77,65 @@ struct XServerLocalPrivate
 
 G_DEFINE_TYPE (XServerLocal, x_server_local, X_SERVER_TYPE);
 
+static gchar *version = NULL;
+static guint version_major = 0, version_minor = 0;
 static GList *display_numbers = NULL;
 
+#define XORG_VERSION_PREFIX "X.Org X Server "
+
+static gchar *
+find_version (const gchar *line)
+{
+    if (!g_str_has_prefix (line, XORG_VERSION_PREFIX))
+        return NULL;
+
+    return g_strdup (line + strlen (XORG_VERSION_PREFIX));
+}
+
+const gchar *
+x_server_local_get_version (void)
+{
+    gchar *stderr_text;
+    gint exit_status;
+    gchar **tokens;
+    guint n_tokens;
+
+    if (version)
+        return version;
+
+    if (!g_spawn_command_line_sync ("X -version", NULL, &stderr_text, &exit_status, NULL))
+        return NULL;
+    if (exit_status == EXIT_SUCCESS)
+    {
+        gchar **lines;
+        int i;
+
+        lines = g_strsplit (stderr_text, "\n", -1);
+        for (i = 0; lines[i] && !version; i++)
+            version = find_version (lines[i]);
+        g_strfreev (lines);
+    }
+    g_free (stderr_text);
+
+    tokens = g_strsplit (version, ".", 3);
+    n_tokens = g_strv_length (tokens);
+    version_major = n_tokens > 0 ? atoi (tokens[0]) : 0;
+    version_minor = n_tokens > 1 ? atoi (tokens[1]) : 0;
+    g_strfreev (tokens);
+
+    return version;
+}
+
+gint
+x_server_local_version_compare (guint major, guint minor)
+{
+    x_server_local_get_version ();
+    if (major == version_major)
+        return version_minor - minor;
+    else
+        return version_major - major;
+}
+
 static gboolean
 display_number_in_use (guint display_number)
 {
@@ -140,7 +197,7 @@ x_server_local_release_display_number (guint display_number)
         guint number = GPOINTER_TO_UINT (link->data);
         if (number == display_number)
         {
-            display_numbers = g_list_remove_link (display_numbers, link);
+            display_numbers = g_list_delete_link (display_numbers, link);
             return;
         }
     }
@@ -149,15 +206,20 @@ x_server_local_release_display_number (guint display_number)
 XServerLocal *
 x_server_local_new (void)
 {
-    XServerLocal *self = g_object_new (X_SERVER_LOCAL_TYPE, NULL);
+    XServerLocal *self;
     gchar hostname[1024], *number, *name;
+    XAuthority *cookie;
+
+    self = g_object_new (X_SERVER_LOCAL_TYPE, NULL);
 
     x_server_set_display_number (X_SERVER (self), x_server_local_get_unused_display_number ());
 
     gethostname (hostname, 1024);
     number = g_strdup_printf ("%d", x_server_get_display_number (X_SERVER (self)));
-    x_server_set_authority (X_SERVER (self), x_authority_new_cookie (XAUTH_FAMILY_LOCAL, (guint8*) hostname, strlen (hostname), number));
+    cookie = x_authority_new_cookie (XAUTH_FAMILY_LOCAL, (guint8*) hostname, strlen (hostname), number);
+    x_server_set_authority (X_SERVER (self), cookie);
     g_free (number);
+    g_object_unref (cookie);
 
     name = g_strdup_printf ("x-%d", x_server_get_display_number (X_SERVER (self)));
     display_server_set_name (DISPLAY_SERVER (self), name);
@@ -213,6 +275,13 @@ x_server_local_set_xdg_seat (XServerLocal *server, const gchar *xdg_seat)
     server->priv->xdg_seat = g_strdup (xdg_seat);
 }
 
+void
+x_server_local_set_sharevts (XServerLocal *server, gboolean sharevts)
+{
+    g_return_if_fail (server != NULL);
+    server->priv->sharevts = sharevts;
+}
+
 void
 x_server_local_set_allow_tcp (XServerLocal *server, gboolean allow_tcp)
 {
@@ -326,7 +395,7 @@ get_absolute_command (const gchar *command)
 }
 
 static void
-run_cb (Process *process, XServerLocal *server)
+run_cb (Process *process, gpointer user_data)
 {
     int fd;
 
@@ -335,29 +404,6 @@ run_cb (Process *process, XServerLocal *server)
     dup2 (fd, STDIN_FILENO);
     close (fd);
 
-    /* Redirect output to logfile */
-    if (server->priv->log_file)
-    {
-         int fd;
-         gchar *old_filename;
-
-         /* Move old file out of the way */
-         old_filename = g_strdup_printf ("%s.old", server->priv->log_file);
-         rename (server->priv->log_file, old_filename);
-         g_free (old_filename);
-
-         /* Create new file and log to it */
-         fd = g_open (server->priv->log_file, O_WRONLY | O_CREAT | O_TRUNC, 0600);
-         if (fd < 0)
-             l_warning (server, "Failed to open log file %s: %s", server->priv->log_file, g_strerror (errno));
-         else
-         {
-             dup2 (fd, STDOUT_FILENO);
-             dup2 (fd, STDERR_FILENO);
-             close (fd);
-         }
-    }
-
     /* Set SIGUSR1 to ignore so the X server can indicate it when it is ready */
     signal (SIGUSR1, SIG_IGN);
 }
@@ -438,8 +484,8 @@ static gboolean
 x_server_local_start (DisplayServer *display_server)
 {
     XServerLocal *server = X_SERVER_LOCAL (display_server);
-    gboolean result;
-    gchar *filename, *dir, *absolute_command;
+    gboolean result, backup_logs;
+    gchar *filename, *dir, *log_file, *absolute_command;
     GString *command;
 
     g_return_val_if_fail (server->priv->x_server_process == NULL, FALSE);
@@ -448,17 +494,19 @@ x_server_local_start (DisplayServer *display_server)
 
     g_return_val_if_fail (server->priv->command != NULL, FALSE);
 
-    server->priv->x_server_process = process_new ();
+    server->priv->x_server_process = process_new (run_cb, server);
     process_set_clear_environment (server->priv->x_server_process, TRUE);
-    g_signal_connect (server->priv->x_server_process, "run", G_CALLBACK (run_cb), server);
     g_signal_connect (server->priv->x_server_process, "got-signal", G_CALLBACK (got_signal_cb), server);
     g_signal_connect (server->priv->x_server_process, "stopped", G_CALLBACK (stopped_cb), server);
 
     /* Setup logging */
     filename = g_strdup_printf ("%s.log", display_server_get_name (display_server));
     dir = config_get_string (config_get_instance (), "LightDM", "log-directory");
-    server->priv->log_file = g_build_filename (dir, filename, NULL);
-    l_debug (display_server, "Logging to %s", server->priv->log_file);
+    log_file = g_build_filename (dir, filename, NULL);
+    backup_logs = config_get_boolean (config_get_instance (), "LightDM", "backup-logs");
+    process_set_log_file (server->priv->x_server_process, log_file, TRUE, backup_logs ? LOG_MODE_BACKUP_AND_TRUNCATE : LOG_MODE_APPEND);
+    l_debug (display_server, "Logging to %s", log_file);
+    g_free (log_file);
     g_free (filename);
     g_free (dir);
 
@@ -483,6 +531,9 @@ x_server_local_start (DisplayServer *display_server)
     if (server->priv->xdg_seat)
         g_string_append_printf (command, " -seat %s", server->priv->xdg_seat);
 
+    if (server->priv->sharevts)
+        g_string_append (command, " -sharevts");
+
     write_authority_file (server);
     if (server->priv->authority_file)
         g_string_append_printf (command, " -auth %s", server->priv->authority_file);
@@ -503,7 +554,12 @@ x_server_local_start (DisplayServer *display_server)
         if (server->priv->xdmcp_key)
             g_string_append_printf (command, " -cookie %s", server->priv->xdmcp_key);
     }
-    else if (!server->priv->allow_tcp)
+    else if (server->priv->allow_tcp)
+    {
+        if (x_server_local_version_compare (1, 17) >= 0)
+            g_string_append (command, " -listen tcp");
+    }
+    else
         g_string_append (command, " -nolisten tcp");
 
     if (server->priv->vt >= 0)
@@ -577,10 +633,10 @@ x_server_local_finalize (GObject *object)
         g_signal_handlers_disconnect_matched (self->priv->x_server_process, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
         g_object_unref (self->priv->x_server_process);
     }
-    g_free (self->priv->log_file);
     g_free (self->priv->command);
     g_free (self->priv->config_file);
     g_free (self->priv->layout);
+    g_free (self->priv->xdg_seat);
     g_free (self->priv->xdmcp_server);
     g_free (self->priv->xdmcp_key);
     g_free (self->priv->mir_id);
