@@ -9,8 +9,8 @@
  * license.
  */
 
-#ifndef _XDMCP_SERVER_H_
-#define _XDMCP_SERVER_H_
+#ifndef XDMCP_SERVER_H_
+#define XDMCP_SERVER_H_
 
 #include <glib-object.h>
 
@@ -44,6 +44,10 @@ void xdmcp_server_set_port (XDMCPServer *server, guint port);
 
 guint xdmcp_server_get_port (XDMCPServer *server);
 
+void xdmcp_server_set_listen_address (XDMCPServer *server, const gchar *listen_address);
+
+const gchar *xdmcp_server_get_listen_address (XDMCPServer *server);
+
 void xdmcp_server_set_hostname (XDMCPServer *server, const gchar *hostname);
 
 const gchar *xdmcp_server_get_hostname (XDMCPServer *server);
@@ -58,4 +62,4 @@ gboolean xdmcp_server_start (XDMCPServer *server);
 
 G_END_DECLS
 
-#endif /* _XDMCP_SERVER_H_ */
+#endif /* XDMCP_SERVER_H_ */
