@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Robert Ancell.
+ * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
  * 
  * This program is free software: you can redistribute it and/or modify it under
@@ -27,48 +27,42 @@ xdmcp_session_new (guint16 id)
 guint16
 xdmcp_session_get_id (XDMCPSession *session)
 {
+    g_return_val_if_fail (session != NULL, 0);
     return session->priv->id;
 }
 
 const gchar *
 xdmcp_session_get_manufacturer_display_id (XDMCPSession *session)
 {
+    g_return_val_if_fail (session != NULL, NULL);
     return session->priv->manufacturer_display_id;
 }
 
-const GInetAddress *
+GInetAddress *
 xdmcp_session_get_address (XDMCPSession *session)
 {
+    g_return_val_if_fail (session != NULL, NULL);
     return session->priv->address;
 }
 
-const gchar *
-xdmcp_session_get_authorization_name (XDMCPSession *session)
+XAuthority *
+xdmcp_session_get_authority (XDMCPSession *session)
 {
-    return session->priv->authorization_name;
-}
-
-const guchar *
-xdmcp_session_get_authorization_data (XDMCPSession *session)
-{
-    return session->priv->authorization_data;  
-}
-
-const gsize
-xdmcp_session_get_authorization_data_length (XDMCPSession *session)
-{
-    return session->priv->authorization_data_length;
+    g_return_val_if_fail (session != NULL, NULL);
+    return session->priv->authority;
 }
 
 guint16
 xdmcp_session_get_display_number (XDMCPSession *session)
 {
+    g_return_val_if_fail (session != NULL, 0);
     return session->priv->display_number;
 }
 
 const gchar *
 xdmcp_session_get_display_class (XDMCPSession *session)
 {
+    g_return_val_if_fail (session != NULL, NULL);
     return session->priv->display_class;
 }
 
@@ -77,7 +71,6 @@ xdmcp_session_init (XDMCPSession *session)
 {
     session->priv = G_TYPE_INSTANCE_GET_PRIVATE (session, XDMCP_SESSION_TYPE, XDMCPSessionPrivate);
     session->priv->manufacturer_display_id = g_strdup ("");
-    session->priv->authorization_name = g_strdup ("");
     session->priv->display_class = g_strdup ("");
 }
 
@@ -91,9 +84,8 @@ xdmcp_session_finalize (GObject *object)
     g_free (self->priv->manufacturer_display_id);
     if (self->priv->address)
         g_object_unref (self->priv->address);
-    if (self->priv->address6)
-        g_object_unref (self->priv->address6);
-    g_free (self->priv->authorization_name);
+    if (self->priv->authority)
+        g_object_unref (self->priv->authority);
     g_free (self->priv->display_class);
 
     G_OBJECT_CLASS (xdmcp_session_parent_class)->finalize (object);
