@@ -6,17 +6,16 @@
 
 static GKeyFile *config;
 
-static void
-request_cb (const gchar *message)
-{
-}
-
 int
 main (int argc, char **argv)
 {
-    g_type_init ();
+    GString *status_text;
 
-    status_connect (request_cb);
+#if !defined(GLIB_VERSION_2_36)
+    g_type_init ();
+#endif
+
+    status_connect (NULL, NULL);
 
     config = g_key_file_new ();
     g_key_file_load_from_file (config, g_build_filename (g_getenv ("LIGHTDM_TEST_ROOT"), "script", NULL), G_KEY_FILE_NONE, NULL);
@@ -27,7 +26,12 @@ main (int argc, char **argv)
         return EXIT_FAILURE;
     }
 
-    status_notify ("SCRIPT-HOOK %s", argv[1]);
+    status_text = g_string_new ("SCRIPT-HOOK");
+    g_string_append_printf (status_text, " %s", argv[1]);
+    if (g_getenv ("USER"))
+        g_string_append_printf (status_text, " USER=%s", g_getenv ("USER"));
+    status_notify ("%s", status_text->str);
+    g_string_free (status_text, TRUE);
 
     if (argc > 2)
         return atoi (argv[2]);
