@@ -41,28 +41,21 @@ sigterm_cb (gpointer user_data)
 }
 
 static void
-request_cb (const gchar *request)
+request_cb (const gchar *name, GHashTable *params)
 {
-    gchar *r;
-
-    if (!request)
+    if (!name)
     {
         g_main_loop_quit (loop);
         return;
     }
   
-    r = g_strdup_printf ("%s LOGOUT", session_id);
-    if (strcmp (request, r) == 0)
+    if (strcmp (name, "LOGOUT") == 0)
         exit (EXIT_SUCCESS);
-    g_free (r);
-  
-    r = g_strdup_printf ("%s CRASH", session_id);
-    if (strcmp (request, r) == 0)
-        kill (getpid (), SIGSEGV);
-    g_free (r);
 
-    r = g_strdup_printf ("%s LOCK-SEAT", session_id);
-    if (strcmp (request, r) == 0)
+    else if (strcmp (name, "CRASH") == 0)
+        kill (getpid (), SIGSEGV);
+
+    else if (strcmp (name, "LOCK-SEAT") == 0)
     {
         status_notify ("%s LOCK-SEAT", session_id);
         g_dbus_connection_call_sync (g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL),
@@ -77,10 +70,8 @@ request_cb (const gchar *request)
                                      NULL,
                                      NULL);
     }
-    g_free (r);
 
-    r = g_strdup_printf ("%s LOCK-SESSION", session_id);
-    if (strcmp (request, r) == 0)
+    else if (strcmp (name, "LOCK-SESSION") == 0)
     {
         status_notify ("%s LOCK-SESSION", session_id);
         g_dbus_connection_call_sync (g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL),
@@ -95,10 +86,8 @@ request_cb (const gchar *request)
                                      NULL,
                                      NULL);
     }
-    g_free (r);
 
-    r = g_strdup_printf ("%s LIST-GROUPS", session_id);
-    if (strcmp (request, r) == 0)
+    else if (strcmp (name, "LIST-GROUPS") == 0)
     {
         int n_groups, i;
         gid_t *groups;
@@ -130,47 +119,37 @@ request_cb (const gchar *request)
         free (groups);
     }
 
-    r = g_strdup_printf ("%s READ-ENV NAME=", session_id);
-    if (g_str_has_prefix (request, r))
+    else if (strcmp (name, "READ-ENV") == 0)
     {
-        const gchar *name = request + strlen (r);
+        const gchar *name = g_hash_table_lookup (params, "NAME");
         const gchar *value = g_getenv (name);
         status_notify ("%s READ-ENV NAME=%s VALUE=%s", session_id, name, value ? value : "");
     }
-    g_free (r);
 
-    r = g_strdup_printf ("%s WRITE-STDOUT TEXT=", session_id);
-    if (g_str_has_prefix (request, r))
-        g_print ("%s", request + strlen (r));
-    g_free (r);
+    else if (strcmp (name, "WRITE-STDOUT") == 0)
+        g_print ("%s", (const gchar *) g_hash_table_lookup (params, "TEXT"));
 
-    r = g_strdup_printf ("%s WRITE-STDERR TEXT=", session_id);
-    if (g_str_has_prefix (request, r))
-        g_printerr ("%s", request + strlen (r));
-    g_free (r);
+    else if (strcmp (name, "WRITE-STDERR") == 0)
+        g_printerr ("%s", (const gchar *) g_hash_table_lookup (params, "TEXT"));
 
-    r = g_strdup_printf ("%s READ FILE=", session_id);
-    if (g_str_has_prefix (request, r))
+    else if (strcmp (name, "READ") == 0)
     {
-        const gchar *name = request + strlen (r);
-        gchar *contents;
+        const gchar *name = g_hash_table_lookup (params, "FILE");
+        gchar *contents = NULL;
         GError *error = NULL;
 
         if (g_file_get_contents (name, &contents, NULL, &error))
             status_notify ("%s READ FILE=%s TEXT=%s", session_id, name, contents);
         else
             status_notify ("%s READ FILE=%s ERROR=%s", session_id, name, error->message);
+        g_free (contents);
         g_clear_error (&error);
     }
-    g_free (r);
 
-    r = g_strdup_printf ("%s LIST-UNKNOWN-FILE-DESCRIPTORS", session_id);
-    if (strcmp (request, r) == 0)
+    else if (strcmp (name, "LIST-UNKNOWN-FILE-DESCRIPTORS") == 0)
         status_notify ("%s LIST-UNKNOWN-FILE-DESCRIPTORS FDS=%s", session_id, open_fds->str);
-    g_free (r);
 
-    r = g_strdup_printf ("%s CHECK-X-AUTHORITY", session_id);
-    if (strcmp (request, r) == 0)
+    else if (strcmp (name, "CHECK-X-AUTHORITY") == 0)
     {
         gchar *xauthority;
         GStatBuf file_info;
@@ -196,13 +175,61 @@ request_cb (const gchar *request)
         status_notify ("%s CHECK-X-AUTHORITY MODE=%s", session_id, mode_string->str);
         g_string_free (mode_string, TRUE);
     }
-    g_free (r);
+
+    else if (strcmp (name, "WRITE-SHARED-DATA") == 0)
+    {
+        const gchar *data = g_hash_table_lookup (params, "DATA");
+        gchar *dir;
+      
+        dir = getenv ("XDG_GREETER_DATA_DIR");
+        if (dir)
+        {
+            gchar *path;
+            FILE *f;
+
+            path = g_build_filename (dir, "data", NULL);
+            if (!(f = fopen (path, "w")) || fprintf (f, "%s", data) < 0)
+                status_notify ("%s WRITE-SHARED-DATA ERROR=%s", session_id, strerror (errno));
+            else
+                status_notify ("%s WRITE-SHARED-DATA RESULT=TRUE", session_id);
+
+            if (f)
+                fclose (f);
+            g_free (path);
+        }
+        else
+            status_notify ("%s WRITE-SHARED-DATA ERROR=NO_XDG_GREETER_DATA_DIR", session_id);
+    }
+
+    else if (strcmp (name, "READ-SHARED-DATA") == 0)
+    {
+        gchar *dir;
+
+        dir = getenv ("XDG_GREETER_DATA_DIR");
+        if (dir)
+        {
+            gchar *path;
+            gchar *contents = NULL;
+            GError *error = NULL;
+
+            path = g_build_filename (dir, "data", NULL);
+            if (g_file_get_contents (path, &contents, NULL, &error))
+                status_notify ("%s READ-SHARED-DATA DATA=%s", session_id, contents);
+            else
+                status_notify ("%s WRITE-SHARED-DATA ERROR=%s", session_id, error->message);
+            g_free (path);
+            g_free (contents);
+            g_clear_error (&error);
+        }
+        else
+            status_notify ("%s WRITE-SHARED-DATA ERROR=NO_XDG_GREETER_DATA_DIR", session_id);
+    }
 }
 
 int
 main (int argc, char **argv)
 {
-    gchar *display, *xdg_seat, *xdg_vtnr, *xdg_current_desktop, *xdg_session_cookie, *xdg_session_class, *desktop_session, *mir_socket, *mir_vt, *mir_id;
+    gchar *display, *xdg_seat, *xdg_vtnr, *xdg_current_desktop, *xdg_greeter_data_dir, *xdg_session_cookie, *xdg_session_class, *desktop_session, *mir_socket, *mir_vt, *mir_id;
     GString *status_text;
     int fd, open_max;
 
@@ -210,12 +237,13 @@ main (int argc, char **argv)
     xdg_seat = getenv ("XDG_SEAT");
     xdg_vtnr = getenv ("XDG_VTNR");
     xdg_current_desktop = getenv ("XDG_CURRENT_DESKTOP");
+    xdg_greeter_data_dir = getenv ("XDG_GREETER_DATA_DIR");
     xdg_session_cookie = getenv ("XDG_SESSION_COOKIE");
     xdg_session_class = getenv ("XDG_SESSION_CLASS");
     desktop_session = getenv ("DESKTOP_SESSION");
     mir_socket = getenv ("MIR_SOCKET");
     mir_vt = getenv ("MIR_SERVER_VT");
-    mir_id = getenv ("MIR_ID");
+    mir_id = getenv ("MIR_SERVER_NAME");
     if (display)
     {
         if (display[0] == ':')
@@ -249,7 +277,7 @@ main (int argc, char **argv)
     g_unix_signal_add (SIGINT, sigint_cb, NULL);
     g_unix_signal_add (SIGTERM, sigterm_cb, NULL);
 
-    status_connect (request_cb);
+    status_connect (request_cb, session_id);
 
     status_text = g_string_new ("");
     g_string_printf (status_text, "%s START", session_id);
@@ -259,6 +287,8 @@ main (int argc, char **argv)
         g_string_append_printf (status_text, " XDG_VTNR=%s", xdg_vtnr);
     if (xdg_current_desktop)
         g_string_append_printf (status_text, " XDG_CURRENT_DESKTOP=%s", xdg_current_desktop);
+    if (xdg_greeter_data_dir)
+        g_string_append_printf (status_text, " XDG_GREETER_DATA_DIR=%s", xdg_greeter_data_dir);
     if (xdg_session_cookie)
         g_string_append_printf (status_text, " XDG_SESSION_COOKIE=%s", xdg_session_cookie);
     if (xdg_session_class)
@@ -270,7 +300,7 @@ main (int argc, char **argv)
     if (argc > 1)
         g_string_append_printf (status_text, " NAME=%s", argv[1]);
     g_string_append_printf (status_text, " USER=%s", getenv ("USER"));
-    status_notify (status_text->str);
+    status_notify ("%s", status_text->str);
     g_string_free (status_text, TRUE);
 
     config = g_key_file_new ();
